#!perl 
# Copyright (c) 2006 Yahoo! Inc.  All rights reserved.  
# The copyrights to the contents of this file are licensed under the Perl Artistic License (ver. 15 Aug 1997) 

use strict; use warnings;

use Test::More skip_all => 'user management service not available in the sandbox';

#use SOAP::Lite +trace => [qw/ debug method fault /];
#use lib 't/lib';
#use Yahoo::Marketing::Test::UserManagementService;
#
#Test::Class->runtests;

